package gs.smolban.model

import gs.uuid.v0.UUID
import gs.slug.v0.Slug
import cats.Show

/**
  * Groups are the basic unit of organization in Smolban. Each [[Ticket]] 
  * belongs to a single `Group`.
  *
  * @param id The unique identifier for the group.
  * @param slug The unique slug for the group.
  * @param createdAt The instant at which this group was created.
  */
case class Group(
  id: Group.Id,
  slug: Slug,
  createdAt: CreatedAt
)

object Group:

  /**
   * Unique identifier for a [[Group]]. This is an opaque type for a UUID.
   */
  opaque type Id = UUID

  object Id:

    /**
     * Instantiate a new [[Group.Id]].
     *
     * @param id The underlying UUID.
     * @return The new [[Group.Id]] instance.
     */
    def apply(id: UUID): Id = id

    given CanEqual[Id, Id] = CanEqual.derived

    given Show[Id] = _.toUUID().withoutDashes()

    extension (id: Id)
      /**
       * Unwrap this Group ID.
       *
       * @return The underlying UUID value.
       */
      def toUUID(): UUID = id

  end Id

end Group
